package com.persagy.apm.report.dependencies.dmpalarm.service;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.persagy.apm.common.response.DmpPageResponse;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.dmpalarm.client.DmpAlarmClient;
import com.persagy.apm.report.dependencies.dmpalarm.model.AlarmRecord;
import com.persagy.apm.report.dependencies.dmpalarm.model.dto.AlarmRecordCriteria;
import com.persagy.apm.report.dependencies.dmpalarm.model.dto.AlarmUrlParam;
import com.persagy.apm.report.dependencies.dmpalarm.model.dto.QueryAlarmRecordDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 报警service层
 *
 * @author lixing
 * @version V1.0 2021/8/4 3:32 下午
 **/
@Service
public class AlarmServiceImpl {
    @Autowired
    DmpAlarmClient dmpAlarmClient;
    @Value("${group.code}")
    private String groupCode;

    /**
     * 根据项目id，报警条目和月份查询报警
     *
     * @param projectId 项目id
     * @param itemCodes 报警条目
     * @param month 月份
     * @return 报警记录列表
     * @author lixing
     * @version V1.0 2021/8/4 3:48 下午
     */
    public List<AlarmRecord> queryAlarmByProjectIdAndItemCodesAndMonth(
            String projectId, List<String> itemCodes, Date month) {
        // 查询条件 params
        AlarmUrlParam alarmUrlParam = new AlarmUrlParam(DataConstants.USER_ID, groupCode, projectId, "0");
        // 查询条件 body
        AlarmRecordCriteria alarmRecordCriteria = new AlarmRecordCriteria();
        alarmRecordCriteria.setItemCode(itemCodes);
        JSONObject triggerTime = new JSONObject();
        Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(month);
        Date firstDayOfNextMonth = DateUtils.getMonthOff(firstDayOfMonth, 1);
        triggerTime.put("$gte", DateUtils.date2Str(firstDayOfMonth, DateUtils.SDFSECOND));
        triggerTime.put("$lt", DateUtils.date2Str(firstDayOfNextMonth, DateUtils.SDFSECOND));
        alarmRecordCriteria.setTriggerTime(triggerTime);

        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO(alarmRecordCriteria);
        DmpPageResponse<AlarmRecord> response = dmpAlarmClient.
                queryAlarmRecord(alarmUrlParam, queryAlarmRecordDTO);
        if (response == null) {
            return null;
        }
        return response.getData();
    }
}
