package com.persagy.apm.report.detail.paragraphvalue.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.*;
import com.persagy.apm.report.detail.paragraphvalue.model.dto.*;

import java.util.List;

/**
 * 报告段落取值(ReportParagraphValue) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-14 16:59:59 2021-07-14 16:59:59
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportParagraphValueTool {

    ConvertReportParagraphValueTool INSTANCE = Mappers.getMapper(ConvertReportParagraphValueTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportParagraphValue do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-14 16:59:59
     */
    ReportParagraphValueItemVO convert2ResponseItemDTO(ReportParagraphValue reportParagraphValue);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportParagraphValue do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-14 16:59:59
     */
    ReportParagraphValueListItemVO convert2ResponseListItemDTO(ReportParagraphValue reportParagraphValue);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportParagraphValueList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-14 16:59:59
     */
    List<ReportParagraphValueListItemVO> convert2List(List<ReportParagraphValue> reportParagraphValueList);

    /**
     * addDTO转换为do
     *
     * @param addReportParagraphValueDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-14 16:59:59
     */
    ReportParagraphValue convertAddDto2Entity(AddReportParagraphValueDTO addReportParagraphValueDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportParagraphValue          更新前的do对象
     * @param updateReportParagraphValueDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-14 16:59:59
     */
    ReportParagraphValue convertUpdateDto2Entity(@MappingTarget ReportParagraphValue reportParagraphValue, UpdateReportParagraphValueDTO updateReportParagraphValueDTO);
}

