package com.persagy.apm.report.indicator.manualfilling.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.indicator.manualfilling.model.vo.*;
import com.persagy.apm.report.indicator.manualfilling.model.dto.*;

import java.util.List;

/**
 * 数据录入数据(ManualFillingData) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-27 09:52:21
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertManualFillingDataTool {

    ConvertManualFillingDataTool INSTANCE = Mappers.getMapper(ConvertManualFillingDataTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param manualFillingData do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-27 09:52:21
     */
    ManualFillingDataItemVO convert2ResponseItemDTO(ManualFillingData manualFillingData);

    /**
     * do转换为ResponseListItemVO
     *
     * @param manualFillingData do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-27 09:52:21
     */
    ManualFillingDataListItemVO convert2ResponseListItemDTO(ManualFillingData manualFillingData);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param manualFillingDataList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-27 09:52:21
     */
    List<ManualFillingDataListItemVO> convert2List(List<ManualFillingData> manualFillingDataList);

    /**
     * addDTO转换为do
     *
     * @param addManualFillingDataDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-27 09:52:21
     */
    ManualFillingData convertAddDto2Entity(AddManualFillingDataDTO addManualFillingDataDTO);

    /**
     * updateDTO转换为实体
     *
     * @param manualFillingData          更新前的do对象
     * @param updateManualFillingDataDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-27 09:52:21
     */
    ManualFillingData convertUpdateDto2Entity(@MappingTarget ManualFillingData manualFillingData, UpdateManualFillingDataDTO updateManualFillingDataDTO);
}

