package com.persagy.apm.report.paragraphs.service.impl.energybudget;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.indicator.utils.IndicatorUtils;
import com.persagy.apm.report.userconfig.function.model.Function;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.budget.BudgetContentCodes;
import com.persagy.apm.report.indicator.factory.EnergyBudgetFactory;
import com.persagy.apm.report.indicator.model.entity.BudgetDefine;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.service.impl.ProjectParagraphBuilder;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 能耗预算单项目段落建造者
 *
 * @author lixing
 * @version V1.0 2021/7/26 4:52 下午
 **/
@Service
public class ProjectEnergyBudgetBuilder extends ProjectParagraphBuilder {
    @Autowired
    EnergyBudgetFactory energyBudgetFactory;
    @Autowired
    IFunctionService functionService;

    @Override
    public List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        return paragraphContentVO.getSummary();
    }

    @Override
    public List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {

        Set<String> codeSet = Sets.newSet(
                BudgetContentCodes.currentMonth.name(),
                BudgetContentCodes.monthlyBudget.name(),
                BudgetContentCodes.aggregate.name(),
                BudgetContentCodes.annualBudget.name()
        );

        Map<String, AttrValueVO> sum = super.getSum(groupVOList, itemVOList, codeSet);
        List<AttrValueVO> results = super.map2ValueList(sum);

        /* 计算比例 */
        // 月能耗预算比
        AttrValueVO monthlyProportion = energyBudgetFactory.getMonthlyProportion(
                IndicatorUtils.getAttrDoubleValue(sum.get(BudgetContentCodes.currentMonth.name())),
                IndicatorUtils.getAttrDoubleValue(sum.get(BudgetContentCodes.monthlyBudget.name())));
        results.add(monthlyProportion);
        // 年能耗预算比
        AttrValueVO annualProportion = energyBudgetFactory.getAnnualProportion(
                IndicatorUtils.getAttrDoubleValue(sum.get(BudgetContentCodes.aggregate.name())),
                IndicatorUtils.getAttrDoubleValue(sum.get(BudgetContentCodes.annualBudget.name())));
        results.add(annualProportion);

        return results;
    }

    @Override
    public ItemVO getProjectItemValue(String itemId, String projectId, Date reportMonth) {
        // 获取指标的定义，转换为指标对象
        Function function = functionService.queryFunctionDetail(itemId);
        Gson gson = new Gson();
        BudgetDefine budgetDefine;
        try {
            budgetDefine = gson.fromJson(function.getDefine(), BudgetDefine.class);
            if (budgetDefine.getItemId() == null ||
                    budgetDefine.getEnergyType() == null ||
                    budgetDefine.getModelCode() == null) {
                throw new IllegalArgumentException("能耗用量定义配置错误，错误的定义内容为：" + function);
            }
        } catch (JsonSyntaxException e) {
            throw new IllegalArgumentException("能耗用量定义配置错误，错误的定义内容为：" + function);
        }
        // 从指标库中获取指标取值
        return energyBudgetFactory.getBudgetInfo(function, budgetDefine, projectId, reportMonth);
    }

    @Override
    public List<AttrValueVO> getProjectSummary(ParagraphContentVO projectParagraphContent) {
        if (projectParagraphContent == null) {
            return null;
        }
        return projectParagraphContent.getSummary();
    }
}
