package com.persagy.apm.report.paragraphs.service.impl.iot;

import com.alibaba.excel.util.CollectionUtils;
import com.google.gson.Gson;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.iot.AreaIotContentCodes;
import com.persagy.apm.report.indicator.codes.iot.AreaIotHeaderCodes;
import com.persagy.apm.report.indicator.codes.iot.ProjectIotContentCodes;
import com.persagy.apm.report.indicator.factory.IotFactory;
import com.persagy.apm.report.indicator.model.vo.GraphItemVO;
import com.persagy.apm.report.indicator.model.vo.GraphVO;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import com.persagy.apm.report.paragraphs.constants.StandardNames;
import com.persagy.apm.report.paragraphs.service.impl.AreaIndicatorsParagraphBuilder;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.userconfig.statisticsitems.enums.QualifiedEnum;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 多项目数据质量段落建造者
 *
 * @author lixing
 * @version V1.0 2021/8/6 9:53 上午
 **/
@Service
public class AreaIotBuilder extends AreaIndicatorsParagraphBuilder {
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    ProjectIotBuilder projectIotBuilder;
    @Autowired
    IotFactory iotFactory;
    @Autowired
    IFunctionService functionService;
    @Autowired
    IReportOutlineService reportOutlineService;

    @Override
    public ItemVO getIndicatorItem(Date reportDate, List<String> projectIds, String functionId) {
        ItemVO itemVO = new ItemVO();
        List<AttrValueVO> attrs = initItem(itemVO, functionId);

        Set<String> statisticCodeSet = Sets.newSet(
                ProjectIotContentCodes.isQualified.name(),
                ProjectIotContentCodes.currentMonth.name()
        );

        // 总项目数
        Integer projectsCountValue = projectIds.size();
        AttrValueVO projectsCount = new AttrValueVO(
                AreaIotContentCodes.projectsCount.name(),
                String.valueOf(projectsCountValue), AttrValueTypeEnum.number);
        attrs.add(projectsCount);

        // 不合格项目数
        Integer unqualifiedProjectCount = null;
        // 各项目报告月取值
        List<GraphItemVO> currentMonthValueList = new ArrayList<>();
        for (String projectId : projectIds) {
            SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
            if (simpleProjectInfo == null) {
                continue;
            }
            ItemVO projectItemValue = projectIotBuilder.getProjectItemValue(
                    functionId, projectId, reportDate);
            Map<String, AttrValueVO> attrValueMap = attrValueList2Map(statisticCodeSet, projectItemValue.getAttrs());

            // 统计各项目报告月取值
            GraphItemVO graphItemVO = new GraphItemVO(simpleProjectInfo.getProjectLocalName(),
                    DataUtils.parseDouble(attrValueMap.get(ProjectIotContentCodes.currentMonth.name()).getValue()),
                    AttrValueTypeEnum.percent_2, DataConstants.PERCENT);
            currentMonthValueList.add(graphItemVO);
            // 统计不合格项目数
            String isQualifiedDesc = attrValueMap.get(ProjectIotContentCodes.isQualified.name()).getValue();
            if (QualifiedEnum.NOT_QUALIFIED.getDesc().equals(isQualifiedDesc)) {
                unqualifiedProjectCount = unqualifiedProjectCount == null ? 1 : unqualifiedProjectCount + 1;
            }
        }
        // 合格标准
        AttrValueVO qualifiedStandard = iotFactory.getQualifiedStandard(
                functionService.queryFunctionDetail(functionId));
        attrs.add(qualifiedStandard);
        // 不合格项目数
        String unqualifiedProjectCountStr = unqualifiedProjectCount == null ?
                "0" : String.valueOf(unqualifiedProjectCount);
        AttrValueVO unqualifiedCount = new AttrValueVO(
                AreaIotContentCodes.unqualifiedCount.name(),
                unqualifiedProjectCountStr, AttrValueTypeEnum.number);
        attrs.add(unqualifiedCount);
        // 各项目本月取值，按绝对值排序
        sortByAbsValueDesc(currentMonthValueList);
        Gson gson = new Gson();
        GraphVO graphVO = new GraphVO(StandardNames.STANDARD_VALUE,
                qualifiedStandard.getValue(), AttrValueTypeEnum.percent, currentMonthValueList);
        AttrValueVO projectValues = new AttrValueVO(
                AreaIotContentCodes.projectValues.name(),
                gson.toJson(graphVO), AttrValueTypeEnum.graph);
        attrs.add(projectValues);

        return itemVO;
    }

    /**
     * 根据绝对值排序(降序)
     *
     * @param graphItemList 图表条目列表
     * @author lixing
     * @version V1.0 2021/8/9 9:40 上午
     */
    private void sortByAbsValueDesc(List<GraphItemVO> graphItemList) {
        if (CollectionUtils.isEmpty(graphItemList)) {
            return;
        }

        graphItemList.sort((o1, o2) -> {
            if (o1 == null) {
                return -1;
            }
            if (o2 == null) {
                return 1;
            }
            return DataUtils.absComparable().reversed().compare(o1.getValue(), o2.getValue());
        });
    }

    @Override
    public List<AttrValueVO> getHeader(ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        List<AttrValueVO> results = new ArrayList<>();

        // 各指标不合格项目数
        String unqualifiedProjectCount = null;
        // 总项目数
        List<SimpleProjectVO> simpleProjectList = reportOutlineService.queryRelatedProjects(reportOutline);
        String projectCountStr = simpleProjectList == null ? "0" : String.valueOf(simpleProjectList.size());

        List<ItemVO> items = paragraphContentVO.getItems();
        Set<String> attrCodeSet = Sets.newSet(
                AreaIotContentCodes.projectsCount.name(),
                AreaIotContentCodes.unqualifiedCount.name()
        );
        if (!CollectionUtils.isEmpty(items)) {
            for (ItemVO item : items) {
                // 获取每个条目的不合格项目数
                List<AttrValueVO> attrs = item.getAttrs();
                Map<String, AttrValueVO> attrMap = attrValueList2Map(attrCodeSet, attrs);
                String itemUnqualifiedProjectCount = attrMap.get(AreaIotContentCodes.unqualifiedCount.name()).getValue();
                itemUnqualifiedProjectCount = itemUnqualifiedProjectCount == null ? "0" : itemUnqualifiedProjectCount;
                String itemInfo = item.getName() + itemUnqualifiedProjectCount + "个";
                unqualifiedProjectCount = unqualifiedProjectCount == null ?
                        itemInfo : unqualifiedProjectCount + "," + itemInfo;
            }
        }

        // 总项目数
        AttrValueVO projectCountVO = new AttrValueVO(
                AreaIotHeaderCodes.projectsCount.name(), projectCountStr, AttrValueTypeEnum.number);
        results.add(projectCountVO);
        // 各指标不合格项目数
        AttrValueVO unqualifiedProjectCountVO = new AttrValueVO(
                AreaIotHeaderCodes.unqualifiedProjectCount.name(), unqualifiedProjectCount, AttrValueTypeEnum.str);
        results.add(unqualifiedProjectCountVO);

        return results;
    }

    @Override
    public List<AttrValueVO> getSummary(List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        return null;
    }
}
