package com.persagy.apm.report.userconfig.paragraph.service;

import com.persagy.apm.report.userconfig.paragraph.model.*;
import com.persagy.apm.report.userconfig.paragraph.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;

/**
 * 段落(Paragraph) service接口
 *
 * @author lixing
 * @version V1.0 2021-07-15 17:46:12
 */
public interface IParagraphService {

    /**
     * 创建段落
     *
     * @param addParagraphDTO addDTO
     * @return 段落主键
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    String createParagraph(AddParagraphDTO addParagraphDTO);

    /**
     * 段落详情
     *
     * @param id 主键
     * @return 段落do对象
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    Paragraph queryParagraphDetail(String id);

    /**
     * 更新段落
     *
     * @param updateParagraphDTO updateDTO
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    void updateParagraph(UpdateParagraphDTO updateParagraphDTO);

    /**
     * 删除段落
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    void deleteParagraph(String id);

    /**
     * 查询段落
     *
     * @param queryParagraphDTO queryDTO
     * @return List<Paragraph>
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    List<Paragraph> queryParagraphList(QueryParagraphDTO queryParagraphDTO);

    /**
     * 分页查询段落
     *
     * @param pageQueryParagraphDTO pageQueryDTO
     * @return IPage<Paragraph>
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    IPage<Paragraph> pageQueryParagraph(PageQueryParagraphDTO pageQueryParagraphDTO);

    /**
     * 获取段落的模板编码
     *
     * @param paragraphId 段落Id
     * @return 模板编码
     * @author lixing
     * @version V1.0 2021/7/20 10:52 上午
     */
    String getModelCode(String paragraphId);

    /**
     * 获取段落的模板编码
     *
     * @param paragraph 段落对象
     * @return 模板编码
     * @author lixing
     * @version V1.0 2021/7/20 10:52 上午
     */
    String getModelCode(Paragraph paragraph);

    /**
     * 获取段落关联的指标id列表
     *
     * @param paragraph 段落
     * @return 段落关联的指标id列表
     * @author lixing
     * @version V1.0 2021/8/5 11:37 上午
     */
    List<String> getRelItemIds(Paragraph paragraph);
}
