package com.persagy.apm.report.userconfig.paragraphcustom.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.userconfig.paragraphcustom.model.vo.*;
import com.persagy.apm.report.userconfig.paragraphcustom.model.dto.*;

import java.util.List;

/**
 * 段落定制(ParagraphCustom) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-15 18:12:06 2021-07-15 18:12:06
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertParagraphCustomTool {

    ConvertParagraphCustomTool INSTANCE = Mappers.getMapper(ConvertParagraphCustomTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param paragraphCustom do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-15 18:12:06
     */
    ParagraphCustomItemVO convert2ResponseItemDTO(ParagraphCustom paragraphCustom);

    /**
     * do转换为ResponseListItemVO
     *
     * @param paragraphCustom do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-15 18:12:06
     */
    ParagraphCustomListItemVO convert2ResponseListItemDTO(ParagraphCustom paragraphCustom);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param paragraphCustomList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-15 18:12:06
     */
    List<ParagraphCustomListItemVO> convert2List(List<ParagraphCustom> paragraphCustomList);

    /**
     * addDTO转换为do
     *
     * @param addParagraphCustomDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-15 18:12:06
     */
    ParagraphCustom convertAddDto2Entity(AddParagraphCustomDTO addParagraphCustomDTO);

    /**
     * updateDTO转换为实体
     *
     * @param paragraphCustom          更新前的do对象
     * @param updateParagraphCustomDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-15 18:12:06
     */
    ParagraphCustom convertUpdateDto2Entity(@MappingTarget ParagraphCustom paragraphCustom, UpdateParagraphCustomDTO updateParagraphCustomDTO);
}

