package com.persagy.apm.report.userconfig.paragraphcustom.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.report.userconfig.paragraphcustom.dao.ParagraphCustomMapper;
import com.persagy.apm.report.userconfig.paragraphcustom.service.IParagraphCustomService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.report.userconfig.paragraphcustom.model.*;
import com.persagy.apm.report.userconfig.paragraphcustom.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 段落定制(ParagraphCustom) service层
 *
 * @author lixing
 * @version V1.0 2021-07-15 18:12:04
 */
@Service
public class ParagraphCustomServiceImpl extends ServiceImpl<ParagraphCustomMapper, ParagraphCustom>
        implements IParagraphCustomService {

    /**
     * 创建段落定制
     *
     * @return 段落定制主键
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public String createParagraphCustom(AddParagraphCustomDTO addParagraphCustomDTO) {
        ParagraphCustom paragraphCustom = ConvertParagraphCustomTool.INSTANCE.convertAddDto2Entity(addParagraphCustomDTO);
        // 设置默认值
        setDefaultValue(paragraphCustom);
        save(paragraphCustom);
        return paragraphCustom.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param paragraphCustom 段落定制实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ParagraphCustom paragraphCustom) {
        paragraphCustom.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 段落定制详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public ParagraphCustom queryParagraphCustomDetail(String id) {
        ParagraphCustom paragraphCustom = getById(id);
        if (paragraphCustom == null) {
            throw new IllegalArgumentException("查看ParagraphCustom详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return paragraphCustom;
    }

    /**
     * 更新段落定制
     *
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public void updateParagraphCustom(UpdateParagraphCustomDTO updateParagraphCustomDTO) {
        ParagraphCustom paragraphCustom = getById(updateParagraphCustomDTO.getId());
        paragraphCustom = ConvertParagraphCustomTool.INSTANCE.convertUpdateDto2Entity(paragraphCustom, updateParagraphCustomDTO);
        paragraphCustom.setModifier(AppContext.getContext().getAccountId());
        updateById(paragraphCustom);
    }

    /**
     * 校验段落定制是否可删除
     *
     * @param id 段落定制主键
     * @return 段落定制do类
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    public ParagraphCustom checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ParagraphCustom时发生异常，主键为空");
        }

        ParagraphCustom paragraphCustom = getById(id);

        if (paragraphCustom == null) {
            throw new IllegalArgumentException("删除ParagraphCustom时发生异常，找不到要删除的数据，id:" + id);
        }

        return paragraphCustom;
    }

    /**
     * 删除段落定制
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public void deleteParagraphCustom(String id) {
        // 校验是否可删除
        ParagraphCustom paragraphCustom = checkDeletable(id);

        paragraphCustom.setValid(ValidEnum.FALSE.getType());
        updateById(paragraphCustom);
    }

    /**
     * 查询段落定制
     *
     * @return List<ParagraphCustom>
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public List<ParagraphCustom> queryParagraphCustomList(QueryParagraphCustomDTO queryParagraphCustomDTO) {
        QueryWrapper<ParagraphCustom> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphCustom.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ParagraphCustom.PROP_CREATIONTIME);

        if (queryParagraphCustomDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphCustomDTO.getDefineId())) {
                queryWrapper.like(ParagraphCustom.PROP_DEFINE_ID, queryParagraphCustomDTO.getDefineId());
            }

            if (queryParagraphCustomDTO.getHeaderAttrs() != null) {
                queryWrapper.eq(ParagraphCustom.PROP_HEADER_ATTRS, queryParagraphCustomDTO.getHeaderAttrs());
            }

            if (queryParagraphCustomDTO.getListAttrs() != null) {
                queryWrapper.eq(ParagraphCustom.PROP_LIST_ATTRS, queryParagraphCustomDTO.getListAttrs());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询段落定制
     *
     * @return IPage<ParagraphCustom>
     * @author lixing
     * @version V1.0 2021-07-15 18:12:04
     */
    @Override
    public IPage<ParagraphCustom> pageQueryParagraphCustom(PageQueryParagraphCustomDTO pageQueryParagraphCustomDTO) {
        QueryWrapper<ParagraphCustom> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphCustom.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ParagraphCustom> pageParam = new Page<>(pageQueryParagraphCustomDTO.getPage(), pageQueryParagraphCustomDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryParagraphCustomDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ParagraphCustom.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryParagraphCustomDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphCustomDTO.getDefineId())) {
            queryWrapper.like(ParagraphCustom.PROP_DEFINE_ID, pageQueryParagraphCustomDTO.getDefineId());
        }

        if (pageQueryParagraphCustomDTO.getHeaderAttrs() != null) {
            queryWrapper.eq(ParagraphCustom.PROP_HEADER_ATTRS, pageQueryParagraphCustomDTO.getHeaderAttrs());
        }

        if (pageQueryParagraphCustomDTO.getListAttrs() != null) {
            queryWrapper.eq(ParagraphCustom.PROP_LIST_ATTRS, pageQueryParagraphCustomDTO.getListAttrs());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
