package com.persagy.apm.report.userconfig.paragraphmodel.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.report.userconfig.paragraphmodel.dao.ParagraphModelMapper;
import com.persagy.apm.report.userconfig.paragraphmodel.service.IParagraphModelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.report.userconfig.paragraphmodel.model.*;
import com.persagy.apm.report.userconfig.paragraphmodel.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 段落模板(ParagraphModel) service层
 *
 * @author lixing
 * @version V1.0 2021-07-20 10:26:10
 */
@Service
public class ParagraphModelServiceImpl extends ServiceImpl<ParagraphModelMapper, ParagraphModel>
        implements IParagraphModelService {

    /**
     * 创建段落模板
     *
     * @return 段落模板主键
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public String createParagraphModel(AddParagraphModelDTO addParagraphModelDTO) {
        ParagraphModel paragraphModel = ConvertParagraphModelTool.INSTANCE.convertAddDto2Entity(addParagraphModelDTO);
        // 设置默认值
        setDefaultValue(paragraphModel);
        save(paragraphModel);
        return paragraphModel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param paragraphModel 段落模板实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ParagraphModel paragraphModel) {
        paragraphModel.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 段落模板详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public ParagraphModel queryParagraphModelDetail(String id) {
        ParagraphModel paragraphModel = getById(id);
        if (paragraphModel == null) {
            throw new IllegalArgumentException("查看ParagraphModel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return paragraphModel;
    }

    /**
     * 更新段落模板
     *
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public void updateParagraphModel(UpdateParagraphModelDTO updateParagraphModelDTO) {
        ParagraphModel paragraphModel = getById(updateParagraphModelDTO.getId());
        paragraphModel = ConvertParagraphModelTool.INSTANCE.convertUpdateDto2Entity(paragraphModel, updateParagraphModelDTO);
        paragraphModel.setModifier(AppContext.getContext().getAccountId());
        updateById(paragraphModel);
    }

    /**
     * 校验段落模板是否可删除
     *
     * @param id 段落模板主键
     * @return 段落模板do类
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    public ParagraphModel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ParagraphModel时发生异常，主键为空");
        }

        ParagraphModel paragraphModel = getById(id);

        if (paragraphModel == null) {
            throw new IllegalArgumentException("删除ParagraphModel时发生异常，找不到要删除的数据，id:" + id);
        }

        return paragraphModel;
    }

    /**
     * 删除段落模板
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public void deleteParagraphModel(String id) {
        // 校验是否可删除
        ParagraphModel paragraphModel = checkDeletable(id);

        paragraphModel.setValid(ValidEnum.FALSE.getType());
        updateById(paragraphModel);
    }

    /**
     * 查询段落模板
     *
     * @return List<ParagraphModel>
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public List<ParagraphModel> queryParagraphModelList(QueryParagraphModelDTO queryParagraphModelDTO) {
        QueryWrapper<ParagraphModel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphModel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ParagraphModel.PROP_CREATIONTIME);

        if (queryParagraphModelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphModelDTO.getCode())) {
                queryWrapper.like(ParagraphModel.PROP_CODE, queryParagraphModelDTO.getCode());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphModelDTO.getName())) {
                queryWrapper.like(ParagraphModel.PROP_NAME, queryParagraphModelDTO.getName());
            }

            if (queryParagraphModelDTO.getHeaderAvailableAttrs() != null) {
                queryWrapper.eq(ParagraphModel.PROP_HEADER_AVAILABLE_ATTRS, queryParagraphModelDTO.getHeaderAvailableAttrs());
            }

            if (queryParagraphModelDTO.getListAvailableAttrs() != null) {
                queryWrapper.eq(ParagraphModel.PROP_LIST_AVAILABLE_ATTRS, queryParagraphModelDTO.getListAvailableAttrs());
            }

            if (queryParagraphModelDTO.getSupplement() != null) {
                queryWrapper.eq(ParagraphModel.PROP_SUPPLEMENT, queryParagraphModelDTO.getSupplement());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询段落模板
     *
     * @return IPage<ParagraphModel>
     * @author lixing
     * @version V1.0 2021-07-20 10:26:10
     */
    @Override
    public IPage<ParagraphModel> pageQueryParagraphModel(PageQueryParagraphModelDTO pageQueryParagraphModelDTO) {
        QueryWrapper<ParagraphModel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphModel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ParagraphModel> pageParam = new Page<>(pageQueryParagraphModelDTO.getPage(), pageQueryParagraphModelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryParagraphModelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ParagraphModel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryParagraphModelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphModelDTO.getCode())) {
            queryWrapper.like(ParagraphModel.PROP_CODE, pageQueryParagraphModelDTO.getCode());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphModelDTO.getName())) {
            queryWrapper.like(ParagraphModel.PROP_NAME, pageQueryParagraphModelDTO.getName());
        }

        if (pageQueryParagraphModelDTO.getHeaderAvailableAttrs() != null) {
            queryWrapper.eq(ParagraphModel.PROP_HEADER_AVAILABLE_ATTRS, pageQueryParagraphModelDTO.getHeaderAvailableAttrs());
        }

        if (pageQueryParagraphModelDTO.getListAvailableAttrs() != null) {
            queryWrapper.eq(ParagraphModel.PROP_LIST_AVAILABLE_ATTRS, pageQueryParagraphModelDTO.getListAvailableAttrs());
        }

        if (pageQueryParagraphModelDTO.getSupplement() != null) {
            queryWrapper.eq(ParagraphModel.PROP_SUPPLEMENT, pageQueryParagraphModelDTO.getSupplement());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
